package tests;

import entities.Course;
import entities.Student;
import java.util.HashSet;
import java.util.Set;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

public class TestJavaEEPersist {
    
    private EntityManagerFactory emf;
    
    private static int STUDENT_ID1 = 420;
    private static String STD_NAME1 = "Charsoubees";
    private static int STUDENT_ID2 = 421;
    private static String STD_NAME2 = "CharsouIkkis";
    private static int STUDENT_ID3 = 422;
    private static String STD_NAME3 = "CharsouBais";
    
    private static  int MATH_ID = 104;
    private static String MATH_COURSE = "Math";
    private static  int CHEM_ID = 204;
    private static String CHEM_COURSE = "Chemistry";
    
    public TestJavaEEPersist() {
    }
    
    public void test() {
        try {
            emf = Persistence.createEntityManagerFactory("onetomany");
            persistRelationship();
            assertRelationshipPersisted();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private void persistRelationship() {
        EntityManager em = emf.createEntityManager();
        EntityTransaction tx = em.getTransaction();
        
        tx.begin();
        
        Course math_course = createCourse(MATH_ID, MATH_COURSE);
        Course chem_course = createCourse(CHEM_ID, CHEM_COURSE);
        Student studentInMath_1 = createStudent(STUDENT_ID1, STD_NAME1, math_course);
        Student studentInMath_2 = createStudent(STUDENT_ID2, STD_NAME2, math_course);
        Student studentInChem = createStudent(STUDENT_ID3, STD_NAME3, chem_course);
        
        Set mathStudents = new HashSet();
        mathStudents.add(studentInMath_1);
        mathStudents.add(studentInMath_2);
        Set chemStudents = new HashSet();
        chemStudents.add(studentInChem);

        math_course.setStudents(mathStudents);
        chem_course.setStudents(chemStudents);
        
        em.persist(math_course);
        em.persist(chem_course);
        em.persist(studentInChem);
        em.persist(studentInMath_1);
        em.persist(studentInMath_2);
        
        tx.commit();
        em.close();
    }
    
    private void assertRelationshipPersisted() throws Exception {
        EntityManager em = emf.createEntityManager();
        
        Course retrievedMath = em.find(Course.class, MATH_ID);
        Course retrievedChem = em.find(Course.class, CHEM_ID);
        
        if (retrievedMath.getStudents().size() == 2 && retrievedChem.getStudents().size() == 1)
            System.out.println("Relationship persisted Correctly");
        else
            System.out.println("Expected one to many relationship between course and student to have been set. " +
                    " Expected 2 students in Math and 1 in Chem, Actual - " + retrievedMath.getStudents().size() + " - " + retrievedChem.getStudents().size());
    }
    
    private Student createStudent(int id, String name, Course course) {
        Student student = new Student();
        student.setId(id);
        student.setName(name);
        student.setCourse(course);
        return student;
    }
    
    private Course createCourse(int id, String courseName) {
        Course course = new Course();
        course.setId(id);
        course.setName(courseName);
        return course;
    }
    
}
